/*
 * Copyright (C) 2005 Markus KARG (markus-karg@users.sourceforge.net)
 * 
 * This file is part of RA Foundation Classes.
 * 
 * RA Foundation Classes is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * RA Foundation Classes is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * RA Foundation Classes; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package net.sourceforge.rafc.spi;

import javax.resource.spi.work.Work;

/**
 * An abstract implementation of the <code>Work</code> interface.<br>
 * This implementation supplies two convenience implementations:
 * </ul>
 * <code>isReleased()</code>.</li>
 * <li>Process management contract: Descendants can check whether the
 * application server requested a releasing of this instance by calling
 * <code>isReleased()</code>.</li>
 * </ul>
 * 
 * @author Markus KARG (markus-karg@users.sourceforge.net)
 */
public abstract class AbstractWork implements Work {

    /**
     * Whether the application server requested releasing of this work instance.<br>
     * Should be checked within long or possibly infinitely running jobs.
     * 
     * @see #isReleased
     * @see #run()
     */
    private volatile Boolean isReleased = Boolean.FALSE;

    /**
     * Enables a flag to remember that the application server requested
     * releasing of this work instance.
     * 
     * @see #isReleased()
     */
    public void release() {
        synchronized (this.isReleased) {
            this.isReleased = Boolean.TRUE;
        }
    }

    /**
     * Allows descendants to check whether this <code>Work</code> shall be
     * released.
     * 
     * @return whether the application server requested releasing of this
     *         instance.
     * @see #release()
     * @see #isReleased
     */
    protected final boolean isReleased() {
        synchronized (this.isReleased) {
            return this.isReleased.booleanValue();
        }
    }

    /**
     * Descendants will overwrite this method to implemented specific work to be
     * done.<br>
     * Since this default implementation is empty actually, there is no need for
     * descendants to call this default implementation by
     * <code>super.release()</code>.<br>
     * To satisfy the process management contract defined in the JCA
     * specification, long lasting operations like (possibly endless) loops
     * should periodically check whether this instance shall be released.
     * 
     * @see #isReleased()
     */
    public void run() {
        /*
         * Intentionally left blank: Descendants will override without calling
         * this default implementation.
         */
    }

}